@@ -1,7 +1,7 @@
 /* Target-dependent code for the AMDGPU architectures.
 
-   Copyright (C) 2019-2023 Free Software Foundation, Inc.
-   Copyright (C) 2019-2023 Advanced Micro Devices, Inc. All rights reserved.
+   Copyright (C) 2019-2024 Free Software Foundation, Inc.
+   Copyright (C) 2019-2024 Advanced Micro Devices, Inc. All rights reserved.
 
    This file is part of GDB.
 
@@ -18,7 +18,6 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 
 #include "amd-dbgapi-target.h"
 #include "amdgpu-tdep.h"
@@ -30,13 +29,12 @@
 #include "gdbsupport/selftest.h"
 #include "gdbtypes.h"
 #include "inferior.h"
+#include "language.h"
 #include "objfiles.h"
 #include "observable.h"
 #include "producer.h"
 #include "reggroups.h"
-#include "amd-dbgapi-target.h"
-
-#include <regex>
+#include "extract-store-integer.h"
 
 /* Bit mask of the address space information in the core address.  */
 constexpr CORE_ADDR AMDGPU_ADDRESS_SPACE_MASK = 0xff00000000000000;
@@ -303,7 +301,7 @@ amdgcn_arg_placement::alloc_for_struct (struct type *type, int offset)
   while (field_num < type->num_fields ())
     {
       /* Allocation units (which contains bitfields) are packed.  */
-      if (TYPE_FIELD_PACKED (type, field_num))
+      if (type->field (field_num).is_packed ())
 	{
 	  has_non_static_fields = true;
 	  /* The first element of the pack must be byte aligned.  */
@@ -315,9 +313,9 @@ amdgcn_arg_placement::alloc_for_struct (struct type *type, int offset)
 	  int pack_end = field_num;
 	  int pack_size = 0;
 	  while (pack_end < type->num_fields ()
-		 && TYPE_FIELD_PACKED (type, pack_end))
+		 && type->field (pack_end).is_packed ())
 	    {
-	      pack_size += type->field (pack_end).bitsize;
+	      pack_size += type->field (pack_end).bitsize ();
 	      pack_end++;
 	    }
 
@@ -343,7 +341,7 @@ amdgcn_arg_placement::alloc_for_struct (struct type *type, int offset)
 	  struct type *field_type = check_typedef (field.type ());
 
 	  /* Ignore static fields.  */
-	  if (!field_is_static (&type->field (field_num)))
+	  if (!type->field (field_num).is_static ())
 	    {
 	      has_non_static_fields = true;
 	      gdb_assert (field.loc_bitpos () % HOST_CHAR_BIT == 0);
@@ -437,8 +435,8 @@ amdgcn_return_value_load_store (gdbarch *gdbarch, regcache *regcache,
   const int base_regno
     = first_regnum_for_arg_or_return_value (gdbarch, regcache->ptid ());
   const int lanenumber
-    = find_thread_ptid (current_inferior (),
-			regcache->ptid ())->current_simd_lane ();
+    = current_inferior ()->find_thread
+	(regcache->ptid ())->current_simd_lane ();
 
   for (const auto &piece : alloc.allocation ())
     {
@@ -479,57 +477,64 @@ amdgcn_return_value_load_store (gdbarch *gdbarch, regcache *regcache,
    [1] https://llvm.org/docs/AMDGPUUsage.html#non-kernel-functions
 */
 static enum return_value_convention
-amdgpu_return_value (struct gdbarch *gdbarch, struct value *function,
-		     struct type *type, struct regcache *regcache,
-		     gdb_byte *readbuf, const gdb_byte *writebuf)
+amdgpu_return_value_as_value (gdbarch *arch, value *function, type *valtype,
+			      regcache *regcache, value **read_value,
+			      const gdb_byte *writebuf)
 {
-  gdb_assert (gdbarch_bfd_arch_info (gdbarch)->arch == bfd_arch_amdgcn);
+  gdb_assert (gdbarch_bfd_arch_info (arch)->arch == bfd_arch_amdgcn);
+  gdb_byte *readbuf = nullptr;
 
-  type = check_typedef (type);
+  valtype = check_typedef (valtype);
+
+  if (read_value != nullptr)
+    {
+      *read_value = value::allocate (valtype);
+      readbuf = (*read_value)->contents_raw ().data ();
+    }
 
   /* Non-trivial objects are not returned by value.  */
-  if (!language_pass_by_reference (type).trivially_copyable)
+  if (!language_pass_by_reference (valtype).trivially_copyable)
     return RETURN_VALUE_STRUCT_CONVENTION;
 
   /* Struct with flexible array are never returned by value.  */
-  if (has_flexible_array_member (type))
+  if (has_flexible_array_member (valtype))
     return RETURN_VALUE_STRUCT_CONVENTION;
 
-  /* Nothing particular to do for empty strucs.  We still use
+  /* Nothing particular to do for empty structs.  We still use
      RETURN_VALUE_REGISTER_CONVENTION so GDB can display the (empty) value.  */
-  if (type->code () == TYPE_CODE_STRUCT)
+  if (valtype->code () == TYPE_CODE_STRUCT)
     {
       bool has_non_static_fields = false;
-      for (int i = 0; i < type->num_fields (); ++i)
+      for (int i = 0; i < valtype->num_fields (); ++i)
 	has_non_static_fields
-	  = has_non_static_fields || !field_is_static (&type->field (i));
+	  = has_non_static_fields || !valtype->field (i).is_static ();
 
       if (!has_non_static_fields)
 	return RETURN_VALUE_REGISTER_CONVENTION;
     }
 
   /* Types of size under 8 bytes are returned packed in v0-1.  */
-  if (type->length () <= 2 * AMDGCN_VGPR_LEN)
+  if (valtype->length () <= 2 * AMDGCN_VGPR_LEN)
     {
       if (regcache != nullptr)
 	{
 	  /* Pack aggregates, but not scalar types so sign extension can be
 	     done if necessary.  */
-	  const bool pack = (type->code () == TYPE_CODE_STRUCT
-			     ||type->code () == TYPE_CODE_ARRAY);
-	  const amdgcn_arg_placement alloc (type, pack);
+	  const bool pack = (valtype->code () == TYPE_CODE_STRUCT
+			     ||valtype->code () == TYPE_CODE_ARRAY);
+	  const amdgcn_arg_placement alloc (valtype, pack);
 
-	  amdgcn_return_value_load_store (gdbarch, regcache,
+	  amdgcn_return_value_load_store (arch, regcache,
 					  alloc, readbuf, writebuf);
 	}
       return RETURN_VALUE_REGISTER_CONVENTION;
     }
 
-  const amdgcn_arg_placement alloc (type, false);
+  const amdgcn_arg_placement alloc (valtype, false);
   if (alloc.allocation ().size () <= AMDGCN_MAX_NUM_REGS_FOR_ARGS_RET)
     {
       if (regcache != nullptr)
-	amdgcn_return_value_load_store (gdbarch, regcache, alloc,
+	amdgcn_return_value_load_store (arch, regcache, alloc,
 					readbuf, writebuf);
       return RETURN_VALUE_REGISTER_CONVENTION;
     }
@@ -657,7 +662,7 @@ struct amd_dbgapi_register_type_flags : public amd_dbgapi_register_type
   using container_type = std::vector<field>;
   using const_iterator_type = container_type::const_iterator;
 
-  amd_dbgapi_register_type_flags (unsigned int bit_size, gdb::string_view name)
+  amd_dbgapi_register_type_flags (unsigned int bit_size, std::string_view name)
     : amd_dbgapi_register_type (kind::FLAGS,
 				make_lookup_name (bit_size, name)),
       m_bit_size (bit_size),
@@ -690,7 +695,7 @@ struct amd_dbgapi_register_type_flags : public amd_dbgapi_register_type
   const std::string &name () const
   { return m_name; }
 
-  static std::string make_lookup_name (int bits, gdb::string_view name)
+  static std::string make_lookup_name (int bits, std::string_view name)
   {
     std::string res = string_printf ("flags%d_t ", bits);
     res.append (name.data (), name.size ());
@@ -717,7 +722,7 @@ struct amd_dbgapi_register_type_enum : public amd_dbgapi_register_type
   using container_type = std::vector<enumerator>;
   using const_iterator_type = container_type::const_iterator;
 
-  amd_dbgapi_register_type_enum (gdb::string_view name)
+  amd_dbgapi_register_type_enum (std::string_view name)
     : amd_dbgapi_register_type (kind::ENUM, make_lookup_name (name)),
       m_name (name.data (), name.length ())
   {}
@@ -746,7 +751,7 @@ struct amd_dbgapi_register_type_enum : public amd_dbgapi_register_type
   const std::string &name () const
   { return m_name; }
 
-  static std::string make_lookup_name (gdb::string_view name)
+  static std::string make_lookup_name (std::string_view name)
   {
     std::string res = "enum ";
     res.append (name.data (), name.length ());
@@ -769,7 +774,7 @@ using amd_dbgapi_register_type_map
 /* Parse S as a ULONGEST, raise an error on overflow.  */
 
 static ULONGEST
-try_strtoulst (gdb::string_view s)
+try_strtoulst (std::string_view s)
 {
   errno = 0;
   ULONGEST value = strtoulst (s.data (), nullptr, 0);
@@ -785,7 +790,7 @@ try_strtoulst (gdb::string_view s)
 #define WSOPT "[ \t]*"
 
 static const amd_dbgapi_register_type &
-parse_amd_dbgapi_register_type (gdb::string_view type_name,
+parse_amd_dbgapi_register_type (std::string_view type_name,
 				amd_dbgapi_register_type_map &type_map);
 
 
@@ -793,7 +798,7 @@ parse_amd_dbgapi_register_type (gdb::string_view type_name,
 
 static void
 parse_amd_dbgapi_register_type_enum_fields
-  (amd_dbgapi_register_type_enum &enum_type, gdb::string_view fields)
+  (amd_dbgapi_register_type_enum &enum_type, std::string_view fields)
 {
   compiled_regex regex (/* name */
 			"^(" IDENTIFIER ")"
@@ -814,14 +819,14 @@ parse_amd_dbgapi_register_type_enum_fields
       auto sv_from_match = [fields] (const regmatch_t &m)
 	{ return fields.substr (m.rm_so, m.rm_eo - m.rm_so); };
 
-      gdb::string_view name = sv_from_match (matches[1]);
-      gdb::string_view value_str = sv_from_match (matches[2]);
+      std::string_view name = sv_from_match (matches[1]);
+      std::string_view value_str = sv_from_match (matches[2]);
       ULONGEST value = try_strtoulst (value_str);
 
       if (value > std::numeric_limits<uint32_t>::max ())
 	enum_type.set_bit_size (64);
 
-      enum_type.add_enumerator (gdb::to_string (name), value);
+      enum_type.add_enumerator (std::string (name), value);
 
       fields = fields.substr (matches[0].rm_eo);
     }
@@ -832,7 +837,7 @@ parse_amd_dbgapi_register_type_enum_fields
 static void
 parse_amd_dbgapi_register_type_flags_fields
   (amd_dbgapi_register_type_flags &flags_type,
-   int bits, gdb::string_view name, gdb::string_view fields,
+   int bits, std::string_view name, std::string_view fields,
    amd_dbgapi_register_type_map &type_map)
 {
   gdb_assert (bits == 32 || bits == 64);
@@ -859,24 +864,24 @@ parse_amd_dbgapi_register_type_flags_fields
       auto sv_from_match = [fields] (const regmatch_t &m)
 	{ return fields.substr (m.rm_so, m.rm_eo - m.rm_so); };
 
-      gdb::string_view field_type_str = sv_from_match (matches[1]);
-      gdb::string_view field_name = sv_from_match (matches[3]);
-      gdb::string_view pos_begin_str = sv_from_match (matches[4]);
+      std::string_view field_type_str = sv_from_match (matches[1]);
+      std::string_view field_name = sv_from_match (matches[3]);
+      std::string_view pos_begin_str = sv_from_match (matches[4]);
       ULONGEST pos_begin = try_strtoulst (pos_begin_str);
 
       if (field_type_str == "bool")
-	flags_type.add_field (gdb::to_string (field_name), pos_begin, pos_begin,
+	flags_type.add_field (std::string (field_name), pos_begin, pos_begin,
 			      nullptr);
       else
 	{
 	  if (matches[5].rm_so == -1)
 	    error (_("Missing end bit position"));
 
-	  gdb::string_view pos_end_str = sv_from_match (matches[5]);
+	  std::string_view pos_end_str = sv_from_match (matches[5]);
 	  ULONGEST pos_end = try_strtoulst (pos_end_str.substr (1));
 	  const amd_dbgapi_register_type &field_type
 	    = parse_amd_dbgapi_register_type (field_type_str, type_map);
-	  flags_type.add_field (gdb::to_string (field_name), pos_begin, pos_end,
+	  flags_type.add_field (std::string (field_name), pos_begin, pos_end,
 				&field_type);
 	}
 
@@ -887,10 +892,10 @@ parse_amd_dbgapi_register_type_flags_fields
 /* parse_amd_dbgapi_register_type helper for scalars.  */
 
 static const amd_dbgapi_register_type &
-parse_amd_dbgapi_register_type_scalar (gdb::string_view name,
+parse_amd_dbgapi_register_type_scalar (std::string_view name,
 				       amd_dbgapi_register_type_map &type_map)
 {
-  std::string name_str = gdb::to_string (name);
+  std::string name_str (name);
   auto it = type_map.find (name_str);
   if (it != type_map.end ())
     {
@@ -933,27 +938,27 @@ parse_amd_dbgapi_register_type_scalar (gdb::string_view name,
    details about the format.  */
 
 static const amd_dbgapi_register_type &
-parse_amd_dbgapi_register_type (gdb::string_view type_str,
+parse_amd_dbgapi_register_type (std::string_view type_str,
 				amd_dbgapi_register_type_map &type_map)
 {
   size_t pos_open_bracket = type_str.find_last_of ('[');
   auto sv_from_match = [type_str] (const regmatch_t &m)
     { return type_str.substr (m.rm_so, m.rm_eo - m.rm_so); };
 
-  if (pos_open_bracket != gdb::string_view::npos)
+  if (pos_open_bracket != std::string_view::npos)
     {
       /* Vector types.  */
-      gdb::string_view element_type_str
+      std::string_view element_type_str
 	= type_str.substr (0, pos_open_bracket);
       const amd_dbgapi_register_type &element_type
 	= parse_amd_dbgapi_register_type (element_type_str, type_map);
 
       size_t pos_close_bracket = type_str.find_last_of (']');
-      gdb_assert (pos_close_bracket != gdb::string_view::npos);
-      gdb::string_view count_str_view
+      gdb_assert (pos_close_bracket != std::string_view::npos);
+      std::string_view count_str_view
 	= type_str.substr (pos_open_bracket + 1,
 			    pos_close_bracket - pos_open_bracket);
-      std::string count_str = gdb::to_string (count_str_view);
+      std::string count_str (count_str_view);
       unsigned int count = std::stoul (count_str);
 
       std::string lookup_name
@@ -987,9 +992,9 @@ parse_amd_dbgapi_register_type (gdb::string_view type_str,
       if (res == REG_NOMATCH)
 	error (_("Failed to parse flags type string"));
 
-      gdb::string_view flags_keyword = sv_from_match (matches[1]);
+      std::string_view flags_keyword = sv_from_match (matches[1]);
       unsigned int bit_size = flags_keyword == "flags32_t" ? 32 : 64;
-      gdb::string_view name = sv_from_match (matches[2]);
+      std::string_view name = sv_from_match (matches[2]);
       std::string lookup_name
 	= amd_dbgapi_register_type_flags::make_lookup_name (bit_size, name);
       auto existing_type_it = type_map.find (lookup_name);
@@ -999,7 +1004,7 @@ parse_amd_dbgapi_register_type (gdb::string_view type_str,
 	  /* No braces, lookup existing type.  */
 	  if (existing_type_it == type_map.end ())
 	    error (_("reference to unknown type %s."),
-		   gdb::to_string (name).c_str ());
+		   std::string (name).c_str ());
 
 	  if (existing_type_it->second->kind ()
 	      != amd_dbgapi_register_type::kind::FLAGS)
@@ -1012,11 +1017,11 @@ parse_amd_dbgapi_register_type (gdb::string_view type_str,
 	  /* With braces, it's a definition.  */
 	  if (existing_type_it != type_map.end ())
 	    error (_("re-definition of type %s."),
-		   gdb::to_string (name).c_str ());
+		   std::string (name).c_str ());
 
 	  amd_dbgapi_register_type_flags_up flags_type
 	    (new amd_dbgapi_register_type_flags (bit_size, name));
-	  gdb::string_view fields_without_braces = sv_from_match (matches[4]);
+	  std::string_view fields_without_braces = sv_from_match (matches[4]);
 
 	  parse_amd_dbgapi_register_type_flags_fields
 	    (*flags_type, bit_size, name, fields_without_braces, type_map);
@@ -1040,7 +1045,7 @@ parse_amd_dbgapi_register_type (gdb::string_view type_str,
       if (res == REG_NOMATCH)
 	error (_("Failed to parse flags type string"));
 
-      gdb::string_view name = sv_from_match (matches[1]);
+      std::string_view name = sv_from_match (matches[1]);
 
       std::string lookup_name
 	= amd_dbgapi_register_type_enum::make_lookup_name (name);
@@ -1051,7 +1056,7 @@ parse_amd_dbgapi_register_type (gdb::string_view type_str,
 	  /* No braces, lookup existing type.  */
 	  if (existing_type_it == type_map.end ())
 	    error (_("reference to unknown type %s"),
-		   gdb::to_string (name).c_str ());
+		   std::string (name).c_str ());
 
 	  if (existing_type_it->second->kind ()
 	      != amd_dbgapi_register_type::kind::ENUM)
@@ -1064,11 +1069,11 @@ parse_amd_dbgapi_register_type (gdb::string_view type_str,
 	  /* With braces, it's a definition.  */
 	  if (existing_type_it != type_map.end ())
 	    error (_("re-definition of type %s"),
-		   gdb::to_string (name).c_str ());
+		   std::string (name).c_str ());
 
 	  amd_dbgapi_register_type_enum_up enum_type
 	    (new amd_dbgapi_register_type_enum (name));
-	  gdb::string_view fields_without_braces = sv_from_match (matches[3]);
+	  std::string_view fields_without_braces = sv_from_match (matches[3]);
 
 	  parse_amd_dbgapi_register_type_enum_fields
 	    (*enum_type, fields_without_braces);
@@ -1094,7 +1099,8 @@ amd_dbgapi_register_type_to_gdb_type (const amd_dbgapi_register_type &type,
     case amd_dbgapi_register_type::kind::INTEGER:
       {
 	const auto &integer_type
-	  = static_cast<const amd_dbgapi_register_type_integer &> (type);
+	  = gdb::checked_static_cast<const amd_dbgapi_register_type_integer &>
+	      (type);
 	switch (integer_type.bit_size ())
 	  {
 	  case 32:
@@ -1117,7 +1123,8 @@ amd_dbgapi_register_type_to_gdb_type (const amd_dbgapi_register_type &type,
     case amd_dbgapi_register_type::kind::VECTOR:
       {
 	const auto &vector_type
-	  = static_cast<const amd_dbgapi_register_type_vector &> (type);
+	  = gdb::checked_static_cast<const amd_dbgapi_register_type_vector &>
+	      (type);
 	struct type *element_type
 	  = amd_dbgapi_register_type_to_gdb_type (vector_type.element_type (),
 						  gdbarch);
@@ -1136,7 +1143,8 @@ amd_dbgapi_register_type_to_gdb_type (const amd_dbgapi_register_type &type,
     case amd_dbgapi_register_type::kind::FLAGS:
       {
 	const auto &flags_type
-	  = static_cast<const amd_dbgapi_register_type_flags &> (type);
+	  = gdb::checked_static_cast<const amd_dbgapi_register_type_flags &>
+	      (type);
 	struct type *gdb_type
 	  = arch_flags_type (gdbarch, flags_type.name ().c_str (),
 			     flags_type.bit_size ());
@@ -1167,15 +1175,14 @@ amd_dbgapi_register_type_to_gdb_type (const amd_dbgapi_register_type &type,
     case amd_dbgapi_register_type::kind::ENUM:
       {
 	const auto &enum_type
-	  = static_cast<const amd_dbgapi_register_type_enum &> (type);
+	  = gdb::checked_static_cast<const amd_dbgapi_register_type_enum &>
+	      (type);
 	struct type *gdb_type
-	  = arch_type (gdbarch, TYPE_CODE_ENUM, enum_type.bit_size (),
-		       enum_type.name ().c_str ());
+	  = (type_allocator (gdbarch)
+	     .new_type (TYPE_CODE_ENUM, enum_type.bit_size (),
+			enum_type.name ().c_str ()));
 
-	gdb_type->set_num_fields (enum_type.size ());
-	gdb_type->set_fields
-	  ((struct field *) TYPE_ZALLOC (gdb_type, (sizeof (struct field)
-						    * enum_type.size ())));
+	gdb_type->alloc_fields (enum_type.size ());
 	gdb_type->set_is_unsigned (true);
 
 	for (size_t i = 0; i < enum_type.size (); ++i)
@@ -1264,7 +1271,7 @@ struct amdgpu_frame_cache
 };
 
 static amdgpu_frame_cache *
-amdgpu_frame_cache (frame_info_ptr this_frame, void **this_cache)
+amdgpu_frame_cache (const frame_info_ptr &this_frame, void **this_cache)
 {
   if (*this_cache != nullptr)
     return (struct amdgpu_frame_cache *) *this_cache;
@@ -1280,7 +1287,7 @@ amdgpu_frame_cache (frame_info_ptr this_frame, void **this_cache)
 }
 
 static void
-amdgpu_frame_this_id (frame_info_ptr this_frame, void **this_cache,
+amdgpu_frame_this_id (const frame_info_ptr &this_frame, void **this_cache,
 		      frame_id *this_id)
 {
   struct amdgpu_frame_cache *cache
@@ -1298,13 +1305,13 @@ amdgpu_frame_this_id (frame_info_ptr this_frame, void **this_cache,
 }
 
 static frame_id
-amdgpu_dummy_id (struct gdbarch *gdbarch, frame_info_ptr this_frame)
+amdgpu_dummy_id (struct gdbarch *gdbarch, const frame_info_ptr &this_frame)
 {
   return frame_id_build (0, get_frame_pc (this_frame));
 }
 
 static struct value *
-amdgpu_frame_prev_register (frame_info_ptr this_frame, void **this_cache,
+amdgpu_frame_prev_register (const frame_info_ptr &this_frame, void **this_cache,
 			    int regnum)
 {
   return frame_unwind_got_register (this_frame, regnum, regnum);
@@ -1597,7 +1604,7 @@ amdgpu_address_spaces (struct gdbarch *gdbarch)
   return tdep->address_spaces;
 }
 
-static enum address_scope
+static location_scope
 amdgpu_address_scope (struct gdbarch *gdbarch, CORE_ADDR address)
 {
   amd_dbgapi_segment_address_dependency_t segment_address_dependency;
@@ -1630,19 +1637,19 @@ amdgpu_address_scope (struct gdbarch *gdbarch, CORE_ADDR address)
   switch (segment_address_dependency)
     {
     case AMD_DBGAPI_SEGMENT_ADDRESS_DEPENDENCE_LANE:
-      return ADDRESS_SCOPE_LANE;
+      return LOCATION_SCOPE_LANE;
 
     case AMD_DBGAPI_SEGMENT_ADDRESS_DEPENDENCE_WAVE:
-      return ADDRESS_SCOPE_THREAD;
+      return LOCATION_SCOPE_THREAD;
 
     case AMD_DBGAPI_SEGMENT_ADDRESS_DEPENDENCE_PROCESS:
-      return ADDRESS_SCOPE_PROCESS;
+      return LOCATION_SCOPE_INFERIOR;
 
     case AMD_DBGAPI_SEGMENT_ADDRESS_DEPENDENCE_WORKGROUP:
     case AMD_DBGAPI_SEGMENT_ADDRESS_DEPENDENCE_AGENT:
       /* GDB currently doesn't model workgroups or agents as first
 	 class citizens, so the mapping here isn't perfect.  */
-      return ADDRESS_SCOPE_PROCESS;
+      return LOCATION_SCOPE_INFERIOR;
 
     default:
       error (_("unhandled segment address dependency kind"));
@@ -1685,10 +1692,23 @@ amdgpu_get_watchable_aliases (struct gdbarch *gdbarch,
 
   std::vector<addr_range> aliases;
   amd_dbgapi_architecture_id_t architecture_id;
-  if (amd_dbgapi_get_architecture (gdbarch_bfd_arch_info (gdbarch)->mach,
-				   &architecture_id)
-	!= AMD_DBGAPI_STATUS_SUCCESS)
-    error (_("amd_dbgapi_get_architecture failed"));
+  /* If we have a wave, use the wave's arch, otherwise, use the arch matching
+     gdbarch.  */
+  if (ptid != null_ptid)
+    {
+      amd_dbgapi_wave_id_t wave_id = get_amd_dbgapi_wave_id (ptid);
+      if (amd_dbgapi_wave_get_info (wave_id, AMD_DBGAPI_WAVE_INFO_ARCHITECTURE,
+				    sizeof (architecture_id), &architecture_id)
+	  != AMD_DBGAPI_STATUS_SUCCESS)
+	error (_("amd_dbgapi_wave_get_info failed"));
+    }
+  else
+    {
+      if (amd_dbgapi_get_architecture (gdbarch_bfd_arch_info (gdbarch)->mach,
+				       &architecture_id)
+	  != AMD_DBGAPI_STATUS_SUCCESS)
+	error (_("amd_dbgapi_get_architecture failed"));
+    }
 
   /* Get a dbgapi address space id for the original address space.  */
   amd_dbgapi_address_space_id_t dbgapi_from_addr_space_id;
@@ -1750,7 +1770,7 @@ amdgpu_get_watchable_aliases (struct gdbarch *gdbarch,
 static simd_lanes_mask_t
 amdgpu_active_lanes_mask (struct gdbarch *gdbarch, thread_info *tp)
 {
-  gdb_static_assert (sizeof (simd_lanes_mask_t) >= sizeof (uint64_t));
+  static_assert (sizeof (simd_lanes_mask_t) >= sizeof (uint64_t));
 
   uint64_t exec_mask;
   if (wave_get_info (tp, AMD_DBGAPI_WAVE_INFO_EXEC_MASK, exec_mask)
@@ -2035,7 +2055,7 @@ amdgpu_gdbarch_init (struct gdbarch_info info, struct gdbarch_list *arches)
 
   set_gdbarch_dwarf2_reg_to_regnum (gdbarch, amdgpu_dwarf_reg_to_regnum);
 
-  set_gdbarch_return_value (gdbarch, amdgpu_return_value);
+  set_gdbarch_return_value_as_value (gdbarch, amdgpu_return_value_as_value);
 
   /* Register representation.  */
   set_gdbarch_register_name (gdbarch, amdgpu_register_name);
@@ -2182,7 +2202,8 @@ amdgpu_register_type_parse_test ()
 
     gdb_assert (type.kind () == amd_dbgapi_register_type::kind::FLAGS);
 
-    const auto &f = static_cast<const amd_dbgapi_register_type_flags &> (type);
+    const auto &f
+      = gdb::checked_static_cast<const amd_dbgapi_register_type_flags &> (type);
     gdb_assert (f.size () == 23);
 
     /* Check the two "FP_ROUND" fields.  */
@@ -2194,7 +2215,8 @@ amdgpu_register_type_parse_test ()
 		      == amd_dbgapi_register_type::kind::ENUM);
 
 	  const auto &e
-	    = static_cast<const amd_dbgapi_register_type_enum &> (*field.type);
+	    = gdb::checked_static_cast<const amd_dbgapi_register_type_enum &>
+	      (*field.type);
 	  gdb_assert (e.size () == 4);
 	  gdb_assert (e[0].name == "NEAREST_EVEN");
 	  gdb_assert (e[0].value == 0);
@@ -2210,7 +2232,8 @@ amdgpu_register_type_parse_test ()
     gdb_assert (f[22].type->kind () == amd_dbgapi_register_type::kind::INTEGER);
 
     const auto &i
-      = static_cast<const amd_dbgapi_register_type_integer &> (*f[22].type);
+      = gdb::checked_static_cast<const amd_dbgapi_register_type_integer &>
+	  (*f[22].type);
     gdb_assert (i.bit_size () == 32);
     gdb_assert (i.is_unsigned ());
   }
@@ -2224,13 +2247,16 @@ amdgpu_register_type_parse_test ()
 
     gdb_assert (type.kind () == amd_dbgapi_register_type::kind::VECTOR);
 
-    const auto &v = static_cast<const amd_dbgapi_register_type_vector &> (type);
+    const auto &v
+      = gdb::checked_static_cast<const amd_dbgapi_register_type_vector &>
+	  (type);
     gdb_assert (v.count () == 64);
 
     const auto &et = v.element_type ();
     gdb_assert (et.kind () == amd_dbgapi_register_type::kind::INTEGER);
 
-    const auto &i = static_cast<const amd_dbgapi_register_type_integer &> (et);
+    const auto &i
+      = gdb::checked_static_cast<const amd_dbgapi_register_type_integer &> (et);
     gdb_assert (i.bit_size () == 32);
     gdb_assert (!i.is_unsigned ());
   }
