@@ -1,7 +1,7 @@
 /* Convert a DWARF location expression to C
 
-   Copyright (C) 2014-2023 Free Software Foundation, Inc.
-   Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights reserved.
+   Copyright (C) 2014-2024 Free Software Foundation, Inc.
+   Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights reserved.
 
    This file is part of GDB.
 
@@ -18,7 +18,6 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "dwarf2.h"
 #include "objfiles.h"
 #include "dwarf2/expr.h"
@@ -60,6 +59,58 @@ struct insn_info
   unsigned int is_tls : 1;
 };
 
+/* A helper function for compute_stack_depth_worker that handles
+   LLVM extension opcodes for DW_OP_LLVM_USER_ namespace.  This
+   only examines the part of the DWARF expression that describes a
+   LLVM extension.
+
+   OP is a LLVM extension opcode, while the OP_PTR and OP_END are the
+   bounds of the DWARF expression.  */
+
+static const gdb_byte *
+llvm_user_stack_depth (dwarf_llvm_user op, const gdb_byte *op_ptr,
+		       const gdb_byte *op_end, int &stack_depth)
+{
+  switch (op)
+    {
+    case DW_OP_LLVM_USER_offset:
+    case DW_OP_LLVM_USER_bit_offset:
+    case DW_OP_LLVM_USER_form_aspace_address:
+      --stack_depth;
+      break;
+
+    case DW_OP_LLVM_USER_call_frame_entry_reg:
+    case DW_OP_LLVM_USER_undefined:
+    case DW_OP_LLVM_USER_push_lane:
+      ++stack_depth;
+      break;
+
+    case DW_OP_LLVM_USER_extend:
+    case DW_OP_LLVM_USER_select_bit_piece:
+      stack_depth -= 2;
+      break;
+
+    case DW_OP_LLVM_USER_aspace_bregx:
+      {
+	uint64_t reg;
+	int64_t offset;
+
+	op_ptr = safe_read_uleb128 (op_ptr, op_end, &reg);
+	op_ptr = safe_read_sleb128 (op_ptr, op_end, &offset);
+	break;
+      }
+
+    case DW_OP_LLVM_USER_piece_end:
+    case DW_OP_LLVM_USER_offset_constu:
+      return 0;
+
+    default:
+      error (_("unhandled DWARF llvm user op: %s"), get_DW_OP_LLVM_USER_name (op));
+    }
+
+  return op_ptr;
+}
+
 /* A helper function for compute_stack_depth that does the work.  This
    examines the DWARF expression starting from START and computes
    stack effects.
@@ -358,31 +409,35 @@ compute_stack_depth_worker (int start, int *need_tempvar,
 	  (*info)[offset].label = 1;
 	  break;
 
-	case DW_OP_LLVM_offset:
-	case DW_OP_LLVM_bit_offset:
-	case DW_OP_LLVM_form_aspace_address:
-	  --stack_depth;
+	case DW_OP_nop:
 	  break;
 
+	case DW_OP_LLVM_user:
+	  op_ptr = safe_read_uleb128 (op_ptr, op_end, &reg);
+	  op_ptr = llvm_user_stack_depth ((dwarf_llvm_user) reg,
+					  op_ptr, op_end, stack_depth);
+	  break;
+
+	case DW_OP_LLVM_form_aspace_address:
+	case DW_OP_LLVM_offset:
+	case DW_OP_LLVM_offset_constu:
+	case DW_OP_LLVM_bit_offset:
 	case DW_OP_LLVM_call_frame_entry_reg:
 	case DW_OP_LLVM_undefined:
-	case DW_OP_LLVM_push_lane:
-	  ++stack_depth;
-	  break;
-
-	case DW_OP_LLVM_extend:
-	case DW_OP_LLVM_select_bit_piece:
-	  stack_depth -= 2;
-	  break;
-
-	case DW_OP_LLVM_aspace_bregx:
-	  op_ptr = safe_read_uleb128 (op_ptr, op_end, &reg);
-	  op_ptr = safe_read_sleb128 (op_ptr, op_end, &offset);
-	  break;
-
 	case DW_OP_LLVM_piece_end:
-	case DW_OP_LLVM_offset_constu:
-	case DW_OP_nop:
+	case DW_OP_LLVM_select_bit_piece:
+	case DW_OP_LLVM_extend:
+	case DW_OP_LLVM_aspace_bregx:
+	case DW_OP_LLVM_push_lane:
+	  {
+	    unsigned int llvm_user_op = get_DW_OP_LLVM_USER (op);
+
+	    if (!llvm_user_op)
+	      error (_("unhandled DWARF LLVM user op: %u"), llvm_user_op);
+
+	    op_ptr = llvm_user_stack_depth ((dwarf_llvm_user) llvm_user_op,
+					    op_ptr, op_end, stack_depth);
+	  }
 	  break;
 
 	default:
@@ -667,7 +722,7 @@ do_compile_dwarf_expr_to_c (int indent, string_file *stream,
 	       sym->print_name ());
 
       val = read_var_value (sym, NULL, frame);
-      if (VALUE_LVAL (val) != lval_memory)
+      if (val->lval () != lval_memory)
 	error (_("Symbol \"%s\" cannot be used for compilation evaluation "
 		 "as its address has not been found."),
 	       sym->print_name ());
@@ -679,7 +734,7 @@ do_compile_dwarf_expr_to_c (int indent, string_file *stream,
 
       gdb_printf (stream, "%*s%s = %s;\n",
 		  indent, "", result_name,
-		  core_addr_to_string (value_address (val)));
+		  core_addr_to_string (val->address ()));
       gdb_printf (stream, "%*s}\n", indent - 2, "");
       return;
     }
@@ -917,7 +972,7 @@ do_compile_dwarf_expr_to_c (int indent, string_file *stream,
 	    if (!b)
 	      error (_("No block found for address"));
 
-	    framefunc = block_linkage_function (b);
+	    framefunc = b->linkage_function ();
 
 	    if (!framefunc)
 	      error (_("No function found for block"));
